package betterwithmods.module.hardcore.crafting;

import betterwithmods.BWMod;
import betterwithmods.common.BWMItems;
import betterwithmods.module.Feature;
import betterwithmods.util.InvUtils;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.entity.projectile.EntityFishHook;
import net.minecraft.init.Blocks;
import net.minecraft.init.Items;
import net.minecraft.inventory.IInventory;
import net.minecraft.inventory.InventoryCrafting;
import net.minecraft.item.ItemFishingRod;
import net.minecraft.item.ItemStack;
import net.minecraft.item.crafting.IRecipe;
import net.minecraft.item.crafting.Ingredient;
import net.minecraft.nbt.NBTBase;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.EnumHand;
import net.minecraft.util.NonNullList;
import net.minecraft.util.ResourceLocation;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.text.TextComponentTranslation;
import net.minecraft.world.World;
import net.minecraftforge.common.ForgeHooks;
import net.minecraftforge.common.capabilities.Capability;
import net.minecraftforge.common.capabilities.CapabilityInject;
import net.minecraftforge.common.capabilities.CapabilityManager;
import net.minecraftforge.common.capabilities.ICapabilitySerializable;
import net.minecraftforge.event.AttachCapabilitiesEvent;
import net.minecraftforge.event.entity.player.ItemFishedEvent;
import net.minecraftforge.event.entity.player.ItemTooltipEvent;
import net.minecraftforge.event.entity.player.PlayerInteractEvent;
import net.minecraftforge.fml.common.event.FMLPreInitializationEvent;
import net.minecraftforge.fml.common.eventhandler.SubscribeEvent;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;
import net.minecraftforge.oredict.OreDictionary;
import net.minecraftforge.oredict.ShapedOreRecipe;
import net.minecraftforge.registries.IForgeRegistryEntry;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.util.function.Predicate;

/**
 * Created by primetoxinz on 7/23/17.
 */
public class HCFishing extends Feature {

    public static boolean requireBait, restrictToOpenWater;
    public static int minimumWaterDepth;

    private static final ResourceLocation BAITED_FISHING_ROD = new ResourceLocation(BWMod.MODID, "baited_fishing_rod");
    private static final Ingredient BAIT = Ingredient.func_193369_a(new ItemStack(Items.field_151078_bh), new ItemStack(Items.field_151070_bp), new ItemStack(BWMItems.CREEPER_OYSTER), new ItemStack(Items.field_151115_aP, 1, 2), new ItemStack(Items.field_151115_aP, 1, 3));


    @Override
    public void setupConfig() {
        requireBait = loadPropBool("Require Bait", "Change Fishing Rods to require being Baited with certain items to entice fish, they won't nibble without it!", true);
        restrictToOpenWater = loadPropBool("Restrict to Open Water", "Fishing on underground locations won't work, hook must be placed on a water block with line of sight to the sky.", true);
        minimumWaterDepth = loadPropInt("Minimum Water Depth", "If higher than 1, requires bodies of water to have a minimum depth for fishing to be successful.", 3);
    }

    @Override
    public void preInit(FMLPreInitializationEvent event) {
        CapabilityManager.INSTANCE.register(FishingBait.class, new CapabilityFishingRod(), FishingBait::new);
        addHardcoreRecipe(new ShapedOreRecipe(null, new ItemStack(Items.field_151112_aM), "  I", " IS", "I N", 'S', "string", 'I', "stickWood", 'N', "nuggetIron").setMirrored(true).setRegistryName(new ResourceLocation("minecraft", "fishing_rod")));
        addHardcoreRecipe(new BaitingRecipe());
    }

    @Override
    public String getFeatureDescription() {
        return "Change Fishing Rods to require bait and a large enough water source exposed to the sky.";
    }

    @SubscribeEvent
    public void attachCapability(AttachCapabilitiesEvent<ItemStack> event) {
        if (event.getObject().func_77973_b() instanceof ItemFishingRod) {
            event.addCapability(BAITED_FISHING_ROD, new FishingBait());
        }
    }

    @SubscribeEvent
    public void onFished(ItemFishedEvent event) {
        BlockPos hookPos = getHookSurfacePos(event.getHookEntity());
        if (restrictToOpenWater) {
            if (event.getHookEntity().func_130014_f_().func_189649_b(hookPos.func_177958_n(), hookPos.func_177952_p()) > hookPos.func_177956_o() || !isAirBlock(event.getHookEntity().func_130014_f_(), hookPos)) {
                event.setCanceled(true);
                event.getEntityPlayer().func_145747_a(new TextComponentTranslation("bwm.message.needs_open_sky"));
                return;
            }
        }
        if (minimumWaterDepth > 1) {
            for (int i=1; i<=minimumWaterDepth; i++) {
                if (!isWaterBlock(event.getHookEntity().func_130014_f_(), hookPos.func_177982_a(0, (i*-1), 0))) {
                    event.setCanceled(true);
                    event.getEntityPlayer().func_145747_a(new TextComponentTranslation("bwm.message.needs_deep_water"));
                    return;
                }
            }
        }
        if (requireBait) {
            ItemStack stack = getMostRelevantFishingRod(event.getEntityPlayer());
            if (isFishingRod(stack)) {
                FishingBait cap = stack.getCapability(FISHING_ROD_CAP, EnumFacing.UP);
                if (cap.hasBait()) {
                    cap.setBait(false);
                    NBTTagCompound tag = stack.func_77978_p();
                    if (tag != null && tag.func_74764_b("bait")) {
                        tag.func_74757_a("bait", false);
                    }
                }
            }
        }
    }

    @SubscribeEvent
    public void useFishingRod(PlayerInteractEvent.RightClickItem event) {
        if (requireBait) {
            if (isFishingRod(event.getItemStack())) {
                FishingBait cap = event.getItemStack().getCapability(FISHING_ROD_CAP, EnumFacing.UP);
                if (!cap.hasBait()) {
                    event.setCanceled(true);
                    if (!event.getWorld().field_72995_K && (event.getHand() == EnumHand.MAIN_HAND || event.getHand() == EnumHand.OFF_HAND))
                        event.getEntityPlayer().func_145747_a(new TextComponentTranslation("bwm.message.needs_bait"));
                }
            }
        }
    }

    @SideOnly(Side.CLIENT)
    @SubscribeEvent
    public void onTooltip(ItemTooltipEvent event) {
        if (requireBait) {
            ItemStack stack = event.getItemStack();
            if (isFishingRod(stack)) {
                FishingBait cap = event.getItemStack().getCapability(FISHING_ROD_CAP, EnumFacing.UP);
                boolean bait = cap.hasBait();
                String tooltip = bait ? "Baited" : "Unbaited";
                if (!bait) {
                    NBTTagCompound tag = stack.func_77978_p();
                    if (tag != null && tag.func_74764_b("bait")) {
                        tooltip = tag.func_74767_n("bait") ? "Baited" : "Unbaited";
                    }
                }
                event.getToolTip().add(tooltip);
            }
        }
    }

    @Override
    public boolean hasSubscriptions() {
        return true;
    }

    public static boolean isFishingRod(ItemStack stack) {
        return stack.func_77973_b() instanceof ItemFishingRod && stack.hasCapability(FISHING_ROD_CAP, EnumFacing.UP);
    }

    public static ItemStack getMostRelevantFishingRod(EntityPlayer player) {
        ItemStack itemMain = player.func_184614_ca();
        if (isFishingRod(itemMain) && itemMain.getCapability(FISHING_ROD_CAP, EnumFacing.UP).hasBait()) {
            return itemMain;
        }
        else {
            return player.func_184592_cb();
        }
    }

    public static boolean isBaited(ItemStack stack, boolean baited) {
        return isFishingRod(stack) && stack.getCapability(FISHING_ROD_CAP, EnumFacing.UP).hasBait() == baited;
    }

    public static ItemStack setBaited(ItemStack rod, boolean baited) {
        if (rod.hasCapability(FISHING_ROD_CAP, EnumFacing.UP)) {
            FishingBait cap = rod.getCapability(FISHING_ROD_CAP, EnumFacing.UP);
            cap.setBait(baited);
        }
        if (rod.func_77978_p() == null) {
            rod.func_77982_d(new NBTTagCompound());
        }
        NBTTagCompound tag = rod.func_77978_p();
        tag.func_74757_a("bait", baited);
        return new ItemStack(rod.serializeNBT());
    }

    public static BlockPos getHookSurfacePos(EntityFishHook hookEntity) {
        World world = hookEntity.func_130014_f_();
        BlockPos hookPos = hookEntity.func_180425_c();
        int heightOffset = 0;
        while (isWaterBlock(world, hookPos.func_177982_a(0,heightOffset,0)) && (hookPos.func_177956_o() + heightOffset < 255)) {
            heightOffset++;
        }
        return hookPos.func_177982_a(0,heightOffset,0);
    }

    public static boolean isWaterBlock(World world, BlockPos pos) {
        return (world.func_180495_p(pos).func_177230_c() == Blocks.field_150355_j || world.func_180495_p(pos).func_177230_c() == Blocks.field_150358_i);
    }

    public static boolean isAirBlock(World world, BlockPos pos) {
        return (world.func_180495_p(pos).func_177230_c() == Blocks.field_150350_a);
    }

    public class BaitingRecipe extends IForgeRegistryEntry.Impl<IRecipe> implements IRecipe {
        protected ResourceLocation group;
        protected Predicate<ItemStack> isTool;
        protected Ingredient input;

        public BaitingRecipe() {
            this.group = new ResourceLocation("baiting_recipe");
            this.input = BAIT;
            this.isTool = stack -> isBaited(stack, false);
            setRegistryName(this.func_193358_e());
        }

        @Override
        public boolean func_77569_a(InventoryCrafting inv, World worldIn) {
            return isMatch(inv);
        }

        public ItemStack findRod(InventoryCrafting inv) {
            for (int x = 0; x < inv.func_70302_i_(); x++) {
                ItemStack slot = inv.func_70301_a(x);
                if (isTool.test(slot)) {
                    return slot;
                }
            }
            return ItemStack.field_190927_a;
        }

        public boolean isMatch(IInventory inv) {
            boolean hasTool = false, hasInput = false;
            for (int x = 0; x < inv.func_70302_i_(); x++) {
                boolean inRecipe = false;
                ItemStack slot = inv.func_70301_a(x);

                if (!slot.func_190926_b()) {
                    if (isTool.test(slot)) {
                        if (!hasTool) {
                            hasTool = true;
                            inRecipe = true;
                        } else
                            return false;
                    } else if (OreDictionary.containsMatch(true, InvUtils.asNonnullList(input.func_193365_a()), slot)) {
                        if (!hasInput) {
                            hasInput = true;
                            inRecipe = true;
                        } else
                            return false;
                    }
                    if (!inRecipe)
                        return false;
                }
            }
            return hasTool && hasInput;
        }

        public NonNullList<ItemStack> func_179532_b(InventoryCrafting inv) {
            NonNullList<ItemStack> ret = NonNullList.func_191197_a(inv.func_70302_i_(), ItemStack.field_190927_a);
            for (int i = 0; i < ret.size(); i++)
            {
                ret.set(i,ForgeHooks.getContainerItem(inv.func_70301_a(i)));
            }
            return ret;
        }

        @Override
        public ItemStack func_77572_b(InventoryCrafting inv) {
            ItemStack rod = findRod(inv);
            if (!rod.func_190926_b())
                return setBaited(rod.func_77946_l(), true);
            return ItemStack.field_190927_a;
        }

        @Override
        public boolean func_194133_a(int width, int height) {
            return width * height >= 2;
        }

        @Override
        public ItemStack func_77571_b() {
            return ItemStack.field_190927_a;
        }

        @Override
        public String func_193358_e() {
            return group.toString();
        }


        @Override
        public boolean func_192399_d() {
            return true;
        }
    }


    @SuppressWarnings("CanBeFinal")
    @CapabilityInject(FishingBait.class)
    public static Capability<FishingBait> FISHING_ROD_CAP = null;

    public static class CapabilityFishingRod implements Capability.IStorage<FishingBait> {

        @Nullable
        @Override
        public NBTBase writeNBT(Capability<FishingBait> capability, FishingBait instance, EnumFacing side) {
            return instance.serializeNBT();
        }

        @Override
        public void readNBT(Capability<FishingBait> capability, FishingBait instance, EnumFacing side, NBTBase nbt) {
            instance.deserializeNBT((NBTTagCompound) nbt);
        }
    }

    public static class FishingBait implements ICapabilitySerializable<NBTTagCompound> {
        private boolean bait;

        public FishingBait() {

        }

        public boolean hasBait() {
            return bait;
        }

        public void setBait(boolean bait) {
            this.bait = bait;
        }

        @Override
        public boolean hasCapability(@Nonnull Capability<?> capability, @Nullable EnumFacing facing) {
            return capability == FISHING_ROD_CAP;
        }

        @Nullable
        @Override
        public <T> T getCapability(@Nonnull Capability<T> capability, @Nullable EnumFacing facing) {
            if (capability == FISHING_ROD_CAP)
                return FISHING_ROD_CAP.cast(this);
            return null;
        }

        @Override
        public NBTTagCompound serializeNBT() {
            NBTTagCompound tag = new NBTTagCompound();
            tag.func_74757_a("bait", hasBait());
            return tag;
        }

        @Override
        public void deserializeNBT(NBTTagCompound nbt) {
            setBait(nbt.func_74767_n("bait"));
        }
    }
}
